"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EffectsUtil_1 = require("../utils/EffectsUtil");
class ColorEffect {
    constructor(config) {
        this._config = config;
    }
    get color() {
        return this._color;
    }
    setColor(color, pattern, message) {
        this._color = color;
        this._colorFunction = this.getColorFunction(pattern, message);
    }
    calculate(input) {
        return (0, EffectsUtil_1.getRGBAString)(this._colorFunction(input));
    }
    getColorFunction(pattern, message) {
        switch (this.color) {
            case "cyan":
                return () => EffectsUtil_1.CYAN;
            case "flash1":
                return this._config.version === "osrs"
                    ? (input) => this.getFlash(input, EffectsUtil_1.FLASH_1_OSRS)
                    : (input) => this.getFlash(input, EffectsUtil_1.FLASH_1_RS3);
            case "flash2":
                return this._config.version === "osrs"
                    ? (input) => this.getFlash(input, EffectsUtil_1.FLASH_2_OSRS)
                    : (input) => this.getFlash(input, EffectsUtil_1.FLASH_2_RS3);
            case "flash3":
                return this._config.version === "osrs"
                    ? (input) => this.getFlash(input, EffectsUtil_1.FLASH_3_OSRS)
                    : (input) => this.getFlash(input, EffectsUtil_1.FLASH_3_RS3);
            case "glow1":
                return this._config.version === "osrs"
                    ? (input) => this.getGlow(input, EffectsUtil_1.GLOW_1_OSRS)
                    : (input) => this.getGlow(input, EffectsUtil_1.GLOW_1_RS3);
            case "glow2":
                return this._config.version === "osrs"
                    ? (input) => this.getGlow(input, EffectsUtil_1.GLOW_2_OSRS)
                    : (input) => this.getGlow(input, EffectsUtil_1.GLOW_2_RS3);
            case "glow3":
                return this._config.version === "osrs"
                    ? (input) => this.getGlow(input, EffectsUtil_1.GLOW_3_OSRS)
                    : (input) => this.getGlow(input, EffectsUtil_1.GLOW_3_RS3);
            case "green":
                return () => EffectsUtil_1.GREEN;
            case "pattern": {
                const patternColors = this.createPatternColors(pattern, message);
                return (input) => this.getPattern(input, patternColors);
            }
            case "purple":
                return () => EffectsUtil_1.PURPLE;
            case "rainbow": {
                const rainbowColors = this.createRainbowColors(message);
                return (input) => this.getPattern(input, rainbowColors);
            }
            case "red":
                return () => EffectsUtil_1.RED;
            case "white":
                return () => EffectsUtil_1.WHITE;
            case "yellow":
                return () => EffectsUtil_1.YELLOW;
        }
    }
    getFlash({ frame }, flash) {
        return flash[Number(((2 * frame) % this._config.fps) * 2 >= this._config.fps)];
    }
    getGlow({ frame }, glow) {
        const inbetweenFrames = this._config.totalFrames / (glow.length - 1);
        const index = Math.floor(frame / inbetweenFrames);
        const currentColor = glow[index];
        const nextColor = glow[index + 1];
        return this.getInbetweenColor(currentColor, nextColor, frame, inbetweenFrames);
    }
    getPattern({ index }, pattern) {
        return pattern[index];
    }
    createPatternColors(pattern, message) {
        const buckets = this.createPatternBuckets(pattern.length, message.length);
        return pattern.flatMap((character, index) => Array(buckets[index]).fill(EffectsUtil_1.PATTERN_CHARACTER_TO_COLOR_MAP[character]));
    }
    createRainbowColors(message) {
        switch (message.length) {
            case 1:
                return EffectsUtil_1.RAINBOW_1;
            case 2:
                return EffectsUtil_1.RAINBOW_2;
            case 3:
                return EffectsUtil_1.RAINBOW_3;
            case 4:
                return EffectsUtil_1.RAINBOW_4;
            case 5:
                return EffectsUtil_1.RAINBOW_5;
            case 6:
                return EffectsUtil_1.RAINBOW_6;
        }
        const buckets = this.createPatternBuckets(EffectsUtil_1.RAINBOW_7.length - 1, message.length - 1);
        buckets.push(1);
        return buckets
            .map((bucket, bucketsIndex) => {
            const colors = Array(bucket).fill(EffectsUtil_1.WHITE);
            colors[0] = EffectsUtil_1.RAINBOW_7[bucketsIndex];
            return colors;
        })
            .flatMap((bucket, bucketsIndex) => {
            if (bucket.length === 1 || bucketsIndex === buckets.length - 1) {
                return bucket;
            }
            const currentColor = EffectsUtil_1.RAINBOW_7[bucketsIndex];
            const nextColor = EffectsUtil_1.RAINBOW_7[bucketsIndex + 1];
            return bucket.map((b, bucketIndex) => {
                if (bucketIndex === 0) {
                    return b;
                }
                return this.getInbetweenColor(currentColor, nextColor, bucketIndex, bucket.length);
            });
        });
    }
    getInbetweenColor(currentColor, nextColor, step, inbetweenLength) {
        const incrementFactor = (step % inbetweenLength) / inbetweenLength;
        return currentColor.map((color, index) => color + Math.round((nextColor[index] - color) * incrementFactor));
    }
    createPatternBuckets(patternLength, messageLength) {
        const baseBucketSize = Math.floor(messageLength / patternLength);
        const buckets = Array(patternLength).fill(baseBucketSize);
        const remainder = messageLength % patternLength;
        const increment = Math.floor(patternLength / remainder);
        for (let i = 0; i < remainder; ++i) {
            const index = i * increment;
            ++buckets[index];
        }
        return buckets;
    }
}
exports.default = ColorEffect;
